/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : Edge.h                                                   */
/*                                                                       */
/*       Date : November  2, 1997                                        */
/* Updated at : November  9, 1997                                        */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/**
    Denotes a relationship between two consequtive customers.
*/
/*************************************************************************/

//#include "EdgeGraph.h" 

#ifndef _EDGE_H
#define _EDGE_H

#include <iostream.h>

class Edge;

typedef Edge* EdgePtr;
#include "VRPcommons.h" 
#include "EdgeGraph.h" 

class Edge {

  public :

     Edge(EdgeGraph *prob, customerId from, customerId to);
     Edge(EdgeGraph *prob, customerId from, customerId to,vrptime at);
     Edge();
     Edge(const Edge&);

     vrpcost getCost() const;

     inline customerId getStartNode()  const { return from; } ;
     inline customerId getEndNode()    const { return to; } ;

     doesStartFromDepot() const;

     doesEndAtDepot() const; 

     vrpcapacity getStartNodeDemand() const;
     vrpcapacity getEndNodeDemand() const;

     vrptime getStartDate() const ;
     void    setStartDate(vrptime sTime) ;
     vrptime getStartDateSecondNode() const; 
     vrptime getEndDateFirstNode() const; 

     vrptime getEndDate() const ;

     void nicePrint(ostream&) const;
     void nicePrint2(ostream&) const;
     void printTabular(ostream&) const;

     operator==(const Edge&);
     Edge& operator=(const Edge&);
     friend ostream& operator<<(ostream&,const Edge&);

     inline static void print(EdgePtr ePtr, ostream& os=cout)
     { ePtr->nicePrint(os); os << endl; }

     inline EdgeGraph* getEdgeGraph()   { return myGraph; }
     inline vrptime    getWaitingTime() const { return wtime; }

     vrpcost getCostPerDemand() const;
     isValid() const;

  private :
      customerId  from ;
      customerId  to ;
         vrptime  arrivalDate;  // denotes when a vehicle arrives at "from" node
         vrptime  wtime;  
      EdgeGraph*  myGraph;

     void copyFrom(const Edge&);
     void adjustWaitingTime();

}; // end of Class Edge

#endif
